@@ -248,10 +248,11 @@ def Run_Local_Climate_Zone_Classification(FEATURES=None, LCZC=None, LCZC_FILTERE
     ----------
     [imagery_classification.lczc]\n
     Updates:\n
-    <ul>\n
-    <li><b>2016/08/15</b> Automated filtering of Style Place Holders</li>\n
-    <li><b>2016/09/07</b> added cmd line support for kml import</li>\n
-    </ul>\n
+    \n
+    (-) [2016/08/15] Automated filtering of Style Place Holders\n
+    (-) [2016/09/07] added cmd line support for kml import\n
+    \n
+    \n
     \n
     Arguments
     ----------
@@ -296,9 +297,8 @@ def Run_Supervised_Majority_Choice_Image_Classification(FEATURES=None, TRAIN_ARE
     ----------
     [imagery_classification.classify_majority]\n
     The majority choice tool for supervised image classification runs the selected classification tools using standard settings and takes for each pixel of the resulting classification the class that has been identified most often by the individual classifiers.\n
-    \n
-    <i>Random Forest</i> is not selected by default because it generates randomized results with more or less strong differences each time it is run.\n
-    <i>K-Nearest Neighbours</i> and <i>Artifical Neural Network</i> have been excluded from default, because using them might be a bit more time consuming.\n
+    'Random Forest' is not selected by default because it generates randomized results with more or less strong differences each time it is run.\n
+    'K-Nearest Neighbours' and 'Artifical Neural Network' have been excluded from default, because using them might be a bit more time consuming.\n
     \n
     Arguments
     ----------
