@@ -137,7 +137,7 @@ def Run_Line_Simplification(LINES=None, OUTPUT=None, TOLERANCE=None, Verbose=2):
     Refererences:\n
     - Ramer, U. (1972): An iterative procedure for the polygonal approximation of plane curves. Computer Graphics and Image Processing, 1(3), 244-256\n
     - Douglas, D., Peucker, T. (1973): Algorithms for the reduction of the number of points required to represent a digitized line or its caricature. The Canadian Cartographer 10(2), 112-122\n
-    - Polyline Reduction source code at <a target="_blank" href="http://mappinghacks.com/code/PolyLineReduction/">mappinghacks.com</a>\n
+    - Polyline Reduction source code at [mappinghacks.com](http://mappinghacks.com/code/PolyLineReduction/)\n
     Arguments
     ----------
     - LINES [`input shapes`] : Lines. Line or polygon shapefile to simplify.
@@ -241,8 +241,8 @@ def Run_Line_Smoothing(LINES_IN=None, LINES_OUT=None, METHOD=None, SENSITIVITY=N
     Gaussian filtering with shrinkage correction is described by Lowe (1989).\n
     In case the density of line vertices is too high, the 'Line Simplification' tool can be applied first. If the density of line vertices is too low, additional vertices can be inserted by applying the 'Convert Lines to Points' and the 'Convert Points to Line(s)' tools prior to smoothing.\n
     References:\n
-    Lowe, D. (1989): Organization of Smooth Image Curves at Multiple Scales. International Journal of Computer Vision, 3: 119-130. (<a target="_blank" href="http://www.cs.ubc.ca/~lowe/papers/iccv88.pdf">pdf</a>)\n
-    Mansouryar, M. & Hedayati, A. (2012): Smoothing Via Iterative Averaging (SIA) - A Basic Technique for Line Smoothing. International Journal of Computer and Electrical Engineering Vol. 4, No. 3: 307-311. (<a target="_blank" href="http://www.ijcee.org/papers/501-P063.pdf">pdf</a>)\n
+    Lowe, D. (1989): Organization of Smooth Image Curves at Multiple Scales. International Journal of Computer Vision, 3: 119-130. ([pdf](http://www.cs.ubc.ca/~lowe/papers/iccv88.pdf))\n
+    Mansouryar, M. & Hedayati, A. (2012): Smoothing Via Iterative Averaging (SIA) - A Basic Technique for Line Smoothing. International Journal of Computer and Electrical Engineering Vol. 4, No. 3: 307-311. ([pdf](http://www.ijcee.org/papers/501-P063.pdf))\n
     Arguments
     ----------
     - LINES_IN [`input shapes`] : Lines. The input line shapefile to smooth.
@@ -270,12 +270,13 @@ def Run_Line_Smoothing(LINES_IN=None, LINES_OUT=None, METHOD=None, SENSITIVITY=N
         return Tool.Execute(Verbose)
     return False
 
-def Run_Split_Lines_at_Points(LINES=None, SPLIT=None, INTERSECT=None, OUTPUT=None, EPSILON=None, Verbose=2):
+def Run_Split_Lines_at_Points(LINES=None, SPLIT=None, INTERSECT=None, OUTPUT=None, EPSILON=None, MIN_VERTEX_DIST=None, Verbose=2):
     '''
     Split Lines at Points
     ----------
     [shapes_lines.8]\n
     The tool allows one to split lines at certain points. The points must be provided as point shapes.\n
+    The order in which the input lines are stored is retained, i.e. splitted parts are inserted at their original location in the dataset. By using a minimum vertex distance, a splitting close to existing line vertex locations can be avoided.\n
     Arguments
     ----------
     - LINES [`input shapes`] : Lines. The input lines to split.
@@ -283,6 +284,7 @@ def Run_Split_Lines_at_Points(LINES=None, SPLIT=None, INTERSECT=None, OUTPUT=Non
     - INTERSECT [`output shapes`] : Intersection. The splitted lines.
     - OUTPUT [`choice`] : Output. Available Choices: [0] polylines [1] separate lines Default: 1 Choose the output line type, either polylines or separate lines.
     - EPSILON [`floating point number`] : Epsilon. Default: 0.000000 The tolerance used to find the point-line intersections [map units].
+    - MIN_VERTEX_DIST [`floating point number`] : Minimum Vertex Distance. Default: 0.000000 The minimum distance of a point to a line vertex in order to split the line at the point's location [map units].
 
     - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
     Returns
@@ -296,6 +298,7 @@ def Run_Split_Lines_at_Points(LINES=None, SPLIT=None, INTERSECT=None, OUTPUT=Non
         Tool.Set_Output('INTERSECT', INTERSECT)
         Tool.Set_Option('OUTPUT', OUTPUT)
         Tool.Set_Option('EPSILON', EPSILON)
+        Tool.Set_Option('MIN_VERTEX_DIST', MIN_VERTEX_DIST)
         return Tool.Execute(Verbose)
     return False
 
@@ -354,16 +357,17 @@ def Run_Extract_Closed_Lines(LINES_IN=None, LINES_OUT=None, TOLERANCE=None, MAX_
         return Tool.Execute(Verbose)
     return False
 
-def Run_Split_Lines(INPUT=None, OUTPUT=None, DISTRIBUTION=None, LENGTH=None, CAPS_LENGTH=None, NUMBER=None, CAPS_NUMBER=None, Verbose=2):
+def Run_Split_Lines(INPUT=None, OUTPUT=None, INS_POINTS=None, DISTRIBUTION=None, LENGTH=None, CAPS_LENGTH=None, NUMBER=None, CAPS_NUMBER=None, Verbose=2):
     '''
     Split Lines
     ----------
     [shapes_lines.11]\n
-    The tool allows one to split lines into multiple lines. The lines can be split based on a user-defined line length or the given number of divisions of each input line.\n
+    The tool allows one to split lines into multiple lines. The lines can be split based on a user-defined line length or the given number of divisions of each input line. Optionally, a point shapes layer with the points at which the lines were split can be created. This can be used, e.g., to create the midpoint of each polyline by dividing each line into two halves.\n
     Arguments
     ----------
     - INPUT [`input shapes`] : Input Lines. Input line shapefile.
     - OUTPUT [`output shapes`] : Output Lines. Output line shapefile.
+    - INS_POINTS [`output shapes`] : Inserted Points. The points at which the lines were split.
     - DISTRIBUTION [`choice`] : Distribution. Available Choices: [0] by length [1] by number Default: 0 Choose the method how to split the lines, either by a given line length or by the given number of divisions.
     - LENGTH [`floating point number`] : Length. Minimum: 0.000000 Default: 5.000000 Output line length used to split the lines [map units].
     - CAPS_LENGTH [`choice`] : Caps. Available Choices: [0] start full length [1] start remaining length [2] even ends Default: 0
@@ -379,6 +383,7 @@ def Run_Split_Lines(INPUT=None, OUTPUT=None, DISTRIBUTION=None, LENGTH=None, CAP
     if Tool.is_Okay():
         Tool.Set_Input ('INPUT', INPUT)
         Tool.Set_Output('OUTPUT', OUTPUT)
+        Tool.Set_Output('INS_POINTS', INS_POINTS)
         Tool.Set_Option('DISTRIBUTION', DISTRIBUTION)
         Tool.Set_Option('LENGTH', LENGTH)
         Tool.Set_Option('CAPS_LENGTH', CAPS_LENGTH)
@@ -433,3 +438,28 @@ def Run_Flip_Line_Direction(LINES=None, FLIPPED=None, Verbose=2):
         return Tool.Execute(Verbose)
     return False
 
+def Run_Merge_Line_Parts_to_Lines(PARTS=None, LINES=None, EPSILON=None, Verbose=2):
+    '''
+    Merge Line Parts to Lines
+    ----------
+    [shapes_lines.14]\n
+    The tool allows one to merge the parts of multipart lines into lines. Invalid line parts with less than two points are skipped.\n
+    Arguments
+    ----------
+    - PARTS [`input shapes`] : Multipart Lines. The multipart lines.
+    - LINES [`output shapes`] : Lines. The merged lines.
+    - EPSILON [`floating point number`] : Epsilon. Default: 0.000000 The tolerance used to detect connected parts [map units].
+
+    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
+    Returns
+    ----------
+    `boolean` : `True` on success, `False` on failure.
+    '''
+    Tool = Tool_Wrapper('shapes_lines', '14', 'Merge Line Parts to Lines')
+    if Tool.is_Okay():
+        Tool.Set_Input ('PARTS', PARTS)
+        Tool.Set_Output('LINES', LINES)
+        Tool.Set_Option('EPSILON', EPSILON)
+        return Tool.Execute(Verbose)
+    return False
+
