@@ -105,17 +105,19 @@ CSG_Parameters::~CSG_Parameters(void)
 //---------------------------------------------------------
 void CSG_Parameters::_On_Construction(void)
 {
-	m_pOwner		= NULL;
-	m_pTool			= NULL;
-	m_pManager		= &SG_Get_Data_Manager();
+	m_pOwner       = NULL;
+	m_pTool        = NULL;
+	m_pManager     = &SG_Get_Data_Manager();
 
-	m_Parameters	= NULL;
-	m_nParameters	= 0;
+	m_Callback     = NULL;
+	m_bCallback    = true;
 
-	m_Callback		= NULL;
-	m_bCallback		= true;
+	m_Parameters   = NULL;
+	m_nParameters  = 0;
 
-	m_pGrid_System	= NULL;
+	m_pGrid_System = NULL;
+
+	m_pStack       = NULL;
 }
 
 //---------------------------------------------------------
@@ -123,18 +125,17 @@ bool CSG_Parameters::Create(const CSG_Parameters &Parameters)
 {
 	Destroy();
 
-	m_pOwner		= Parameters.m_pOwner;
-	m_pTool			= Parameters.m_pTool;
-	m_pManager		= Parameters.m_pManager;
+	Set_Identifier (Parameters.Get_Identifier ());
+	Set_Name       (Parameters.Get_Name       ());
+	Set_Description(Parameters.Get_Description());
 
-	m_Callback		= Parameters.m_Callback;
-	m_bCallback		= Parameters.m_bCallback;
+	m_pOwner       = Parameters.m_pOwner;
+	m_pTool        = Parameters.m_pTool;
+	m_pManager     = Parameters.m_pManager;
 
-	Set_Identifier	(Parameters.Get_Identifier ());
-	Set_Name		(Parameters.Get_Name       ());
-	Set_Description	(Parameters.Get_Description());
+	m_Callback     = Parameters.m_Callback;
+	m_bCallback    = Parameters.m_bCallback;
 
-	//-----------------------------------------------------
 	for(int i=0; i<Parameters.m_nParameters; i++)
 	{
 		_Add(Parameters.m_Parameters[i]);
@@ -142,7 +143,7 @@ bool CSG_Parameters::Create(const CSG_Parameters &Parameters)
 
 	if( Parameters.m_pGrid_System )
 	{
-		m_pGrid_System	= Get_Parameter(Parameters.m_pGrid_System->Get_Identifier());
+		m_pGrid_System = Get_Parameter(Parameters.m_pGrid_System->Get_Identifier());
 	}
 
 	return( m_nParameters == Parameters.m_nParameters );
@@ -170,7 +171,7 @@ bool CSG_Parameters::Create(void *pOwner, const SG_Char *Name, const SG_Char *De
 {
 	if( Create(Name, Description, Identifier, bGrid_System) )
 	{
-		m_pOwner	= pOwner;
+		m_pOwner = pOwner;
 
 		return( true );
 	}
@@ -181,6 +182,8 @@ bool CSG_Parameters::Create(void *pOwner, const SG_Char *Name, const SG_Char *De
 //---------------------------------------------------------
 void CSG_Parameters::Destroy(void)
 {
+	Pop();
+
 	m_pOwner       = NULL;
 	m_pTool        = NULL;
 	m_pGrid_System = NULL;
@@ -195,6 +198,12 @@ void CSG_Parameters::Destroy(void)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+void CSG_Parameters::Set_Tool(CSG_Tool *pTool)
+{
+	m_pTool = pTool;
+}
+
 //---------------------------------------------------------
 /**
   * Set the responsible data manager for this parameter list.
@@ -205,7 +214,7 @@ void CSG_Parameters::Destroy(void)
 //---------------------------------------------------------
 void CSG_Parameters::Set_Manager(CSG_Data_Manager *pManager)
 {
-	m_pManager		= pManager;
+	m_pManager = pManager;
 
 	for(int i=0; i<Get_Count(); i++)
 	{
@@ -233,6 +242,58 @@ bool CSG_Parameters::Use_Grid_System(void)
 	return( false );
 }
 
+
+//---------------------------------------------------------
+/**
+* Stores the current parameter settings including the data
+* manager to an internal CSG_Parameters copy.
+* Settings and data manager can be restored by a subsequent
+* call to Pop().
+*/
+//---------------------------------------------------------
+bool CSG_Parameters::Push(CSG_Data_Manager *pManager, bool bRestoreDefaults)
+{
+	CSG_Parameters *pStack = m_pStack;
+
+	m_pStack = new CSG_Parameters(*this);
+
+	m_pStack->m_pStack = pStack;
+
+	m_pManager = pManager;
+
+	if( bRestoreDefaults )
+	{
+		Restore_Defaults(true);
+	}
+
+	return( true );
+}
+
+//---------------------------------------------------------
+/**
+* Restores previously pushed parameter settings and data manager.
+*/
+//---------------------------------------------------------
+bool CSG_Parameters::Pop(void)
+{
+	if( m_pStack )
+	{
+		m_pManager = m_pStack->m_pManager;
+
+		Assign_Values(m_pStack);
+
+		CSG_Parameters *pStack = m_pStack->m_pStack;
+
+		delete( m_pStack );
+
+		m_pStack = pStack;
+
+		return( true );
+	}
+
+	return( false );
+}
+
 //---------------------------------------------------------
 /**
 * If parameters are owned by a tool the function returns the
@@ -1305,9 +1366,9 @@ TSG_PFNC_Parameter_Changed CSG_Parameters::Set_Callback_On_Parameter_Changed(TSG
 // Return value is the previous state.
 bool CSG_Parameters::Set_Callback(bool bActive)
 {
-	bool	bPrevious	= m_bCallback;
+	bool bPrevious = m_bCallback;
 
-	m_bCallback	= bActive;
+	m_bCallback = bActive;
 
 	for(int i=0; i<m_nParameters; i++)
 	{
@@ -1325,7 +1386,7 @@ bool CSG_Parameters::_On_Parameter_Changed(CSG_Parameter *pParameter, int Flags)
 {
 	if( m_Callback && m_bCallback )
 	{
-		bool	bCallback	= Set_Callback(false);
+		bool bCallback = Set_Callback(false);
 
 		m_Callback(pParameter, Flags);
 
@@ -1511,16 +1572,14 @@ bool CSG_Parameters::Assign_Parameters(CSG_Parameters *pSource)
 //---------------------------------------------------------
 bool CSG_Parameters::DataObjects_Check(bool bSilent)
 {
-	bool		bResult	= true;
-
-	CSG_String	sError;
+	bool bResult = true; CSG_String sError;
 
 	//-----------------------------------------------------
 	for(int i=0; i<Get_Count(); i++)
 	{
 		if( m_Parameters[i]->Check(bSilent) == false )
 		{
-			bResult	= false;
+			bResult = false;
 
 			sError.Append(CSG_String::Format("\n%s: %s", m_Parameters[i]->Get_Type_Name().c_str(), m_Parameters[i]->Get_Name()));
 		}
@@ -1554,7 +1613,7 @@ bool CSG_Parameters::DataObjects_Create(void)
 		}
 		else if( P.Get_Type() == PARAMETER_TYPE_DataObject_Output )
 		{
-			if( m_pManager || P.asDataObject() == DATAOBJECT_CREATE )
+			if( (m_pManager && has_GUI()) || P.asDataObject() == DATAOBJECT_CREATE )
 			{
 				P.Set_Value(DATAOBJECT_NOTSET);
 			}
@@ -1582,9 +1641,9 @@ bool CSG_Parameters::DataObjects_Create(void)
 		}
 
 		//-------------------------------------------------
-		else if( P.is_DataObject() && P.is_Enabled() == false )
+		else if( P.is_DataObject() && P.is_Enabled() == false && has_GUI() )
 		{
-			if( P.asDataObject() != DATAOBJECT_CREATE && (!m_pManager || !m_pManager->Exists(P.asDataObject())) )
+			if( P.asDataObject() != DATAOBJECT_CREATE && (m_pManager && !m_pManager->Exists(P.asDataObject())) )
 			{
 				P.Set_Value(DATAOBJECT_NOTSET);
 			}
@@ -1630,13 +1689,16 @@ bool CSG_Parameters::DataObjects_Create(void)
 				{
 					if(	P.Get_Parent() && P.Get_Parent()->asGrid_System() && P.Get_Parent()->asGrid_System()->is_Valid() )
 					{
-						if( P.Get_Type() == PARAMETER_TYPE_Grid )
+						CSG_Grid_System System(*P.Get_Parent()->asGrid_System());
+
+						if( P.Get_Type() == PARAMETER_TYPE_Grid  && !System.is_Equal(pObject->asGrid ()->Get_System()) )
 						{
-							pObject->asGrid ()->Create(*P.Get_Parent()->asGrid_System()       , ((CSG_Parameter_Grid  *)&P)->Get_Preferred_Type());
+							pObject->asGrid ()->Create(System       , ((CSG_Parameter_Grid  *)&P)->Get_Preferred_Type());
 						}
-						else
+
+						if( P.Get_Type() == PARAMETER_TYPE_Grids && !System.is_Equal(pObject->asGrids()->Get_System()) )
 						{
-							pObject->asGrids()->Create(*P.Get_Parent()->asGrid_System(), 0, 0., ((CSG_Parameter_Grids *)&P)->Get_Preferred_Type());
+							pObject->asGrids()->Create(System, 0, 0., ((CSG_Parameter_Grids *)&P)->Get_Preferred_Type());
 						}
 					}
 				}
@@ -1680,23 +1742,23 @@ bool CSG_Parameters::DataObjects_Synchronize(void)
 {
 	for(int i=0; i<Get_Count(); i++)
 	{
-		CSG_Parameter	*p	= m_Parameters[i];
+		CSG_Parameter &P = *m_Parameters[i];
 
-		if( p->Get_Type() == PARAMETER_TYPE_Parameters )
+		if( P.Get_Type() == PARAMETER_TYPE_Parameters )
 		{
-			p->asParameters()->DataObjects_Synchronize();
+			P.asParameters()->DataObjects_Synchronize();
 		}
 
 		//-------------------------------------------------
-		else if( p->is_Output() )
+		else if( P.is_Output() )
 		{
-			if( p->is_DataObject() )
+			if( P.is_DataObject() )
 			{
-				CSG_Data_Object	*pObject	= p->asDataObject();
+				CSG_Data_Object *pObject = P.asDataObject();
 
 				if( pObject == DATAOBJECT_CREATE )
 				{
-					p->Set_Value(DATAOBJECT_NOTSET);
+					P.Set_Value(DATAOBJECT_NOTSET);
 				}
 				else if( pObject != DATAOBJECT_NOTSET )
 				{
@@ -1708,13 +1770,18 @@ bool CSG_Parameters::DataObjects_Synchronize(void)
 							delete(pObject);
 						}
 
-						p->Set_Value(DATAOBJECT_NOTSET);
+						P.Set_Value(DATAOBJECT_NOTSET);
 					}
 					else
 					{
-						if( m_pManager && !m_pManager->Exists(pObject) )
+						if( m_pManager )
 						{
 							m_pManager->Add(pObject);
+
+							if( m_pManager == &SG_Get_Data_Manager() )
+							{
+								SG_UI_DataObject_Add(pObject, 0);
+							}
 						}
 
 						SG_UI_DataObject_Update(pObject, SG_UI_DATAOBJECT_UPDATE, NULL);
@@ -1723,15 +1790,20 @@ bool CSG_Parameters::DataObjects_Synchronize(void)
 			}
 
 			//---------------------------------------------
-			else if( p->is_DataObject_List() )
+			else if( P.is_DataObject_List() )
 			{
-				for(int j=0; j<p->asList()->Get_Item_Count(); j++)
+				for(int j=0; j<P.asList()->Get_Item_Count(); j++)
 				{
-					CSG_Data_Object	*pObject	= p->asList()->Get_Item(j);
+					CSG_Data_Object *pObject = P.asList()->Get_Item(j);
 
-					if( m_pManager && !m_pManager->Exists(pObject) )
+					if( m_pManager )
 					{
 						m_pManager->Add(pObject);
+
+						if( m_pManager == &SG_Get_Data_Manager() )
+						{
+							SG_UI_DataObject_Add(pObject, 0);
+						}
 					}
 
 					SG_UI_DataObject_Update(pObject, SG_UI_DATAOBJECT_UPDATE, NULL);
