@@ -54,6 +54,7 @@
 #include "tool_chain.h"
 
 #include <wx/string.h>
+#include <wx/cmdline.h>
 
 
 ///////////////////////////////////////////////////////////
@@ -65,19 +66,18 @@
 //---------------------------------------------------------
 CSG_Tool::CSG_Tool(void)
 {
-	m_ID			= "-1";
-	m_Version		= "1.0";
+	m_ID      = "-1";
+	m_Version = "1.0";
 
-	m_bError_Ignore	= false;
-	m_bExecutes		= false;
-	m_bWithGUI		= true;
+	m_bError_Ignore = false;
+	m_bExecutes     = false;
 
-	m_pParameters	= NULL;
-	m_npParameters	= 0;
+	m_bGUI = SG_UI_Get_Application_Name().Cmp("saga_gui") == 0;
+	m_bCMD = SG_UI_Get_Application_Name().Cmp("saga_cmd") == 0;
 
 	Parameters.Create(this, SG_T("Tool"));
 	Parameters.Set_Callback_On_Parameter_Changed(&_On_Parameter_Changed);
-	Parameters.m_pTool	= this;
+	Parameters.Set_Tool(this);
 
 	Set_Show_Progress(true);
 }
@@ -85,22 +85,9 @@ CSG_Tool::CSG_Tool(void)
 //---------------------------------------------------------
 CSG_Tool::~CSG_Tool(void)
 {
-	if( m_Settings_Stack.Get_Size() > 0 )
+	for(int i=0; i<Get_Parameters_Count(); i++)
 	{
-		for(size_t i=0; i<m_Settings_Stack.Get_uSize(); i++)
-		{
-			delete(((CSG_Parameters **)m_Settings_Stack.Get_Array())[i]);
-		}
-	}
-
-	if( m_pParameters )
-	{
-		for(int i=0; i<m_npParameters; i++)
-		{
-			delete(m_pParameters[i]);
-		}
-
-		SG_Free(m_pParameters);
+		delete(Get_Parameters(i));
 	}
 
 	Destroy();
@@ -204,35 +191,60 @@ CSG_String CSG_Tool::Get_MenuPath(bool bSolved)
 		return( Get_MenuPath() );
 	}
 
-	CSG_String	Menu	= Get_MenuPath();
+	CSG_Strings Menu = SG_String_Tokenize(Get_MenuPath(), ";"); if( !Menu.Get_Count() ) { Menu += ""; }
 
-	if( Menu.Length() > 1 && Menu[1] == ':' )
+	for(int i=0; i<Menu.Get_Count(); i++)
 	{
-		if( Menu[0] == 'A' || Menu[0] == 'a' )	// absolute menu path, overwrites library's default menu path
+		if( Menu[i].Length() > 1 && Menu[i][1] == ':' )
 		{
-			return( Menu.AfterFirst(':') );
+			if( Menu[i][0] == 'A' || Menu[i][0] == 'a' ) // absolute menu path, overwrites library's default menu path
+			{
+				Menu[i] = Menu[i].AfterFirst(':');
+
+				continue;
+			}
+
+			Menu[i] = Menu[i].AfterFirst(':'); // Menu[0] == 'R' || Menu[0] == 'r' // menu path explicitly declared as relative to library's default menu path
 		}
 
-		Menu	= Menu.AfterFirst(':');	// Menu[0] == 'R' || Menu[0] == 'r'	// menu path explicitly declared as relative to library's default menu path
+		if( !m_Library_Menu.is_Empty() )
+		{
+			if( Menu[i].is_Empty() )
+			{
+				Menu[i] = m_Library_Menu;
+			}
+			else
+			{
+				Menu[i] = m_Library_Menu + "|" + Menu[i];
+			}
+		}
 	}
 
-	if( m_Library_Menu.is_Empty() )
+	CSG_String Menus; 
+
+	for(int i=0; i<Menu.Get_Count(); i++)
 	{
-		return( Menu );
+		if( i > 0 )
+		{
+			Menus += ";";
+		}
+
+		Menus += Menu[i];
 	}
 
-	if( Menu.is_Empty() )
-	{
-		return( m_Library_Menu );
-	}
-
-	return( m_Library_Menu + "|" + Menu );
+	return( Menus );
 }
 
 //---------------------------------------------------------
 bool CSG_Tool::has_GUI(void) const
 {
-	return( m_bWithGUI && SG_UI_Get_Window_Main() != NULL );
+	return( m_bGUI );
+}
+
+//---------------------------------------------------------
+bool CSG_Tool::has_CMD(void) const
+{
+	return( m_bCMD );
 }
 
 
@@ -392,20 +404,20 @@ bool CSG_Tool::_Synchronize_DataObjects(void)
 {
 	Parameters.DataObjects_Synchronize();
 
-	for(int i=0; i<m_npParameters; i++)
+	for(int i=0; i<Get_Parameters_Count(); i++)
 	{
-		m_pParameters[i]->DataObjects_Synchronize();
+		Get_Parameters(i)->DataObjects_Synchronize();
 	}
 
-	CSG_Projection	Projection;
+	CSG_Projection Projection;
 
 	if( do_Sync_Projections() && Get_Projection(Projection) )
 	{
 		Parameters.DataObjects_Set_Projection(Projection);
 
-		for(int j=0; j<m_npParameters; j++)
+		for(int i=0; i<Get_Parameters_Count(); i++)
 		{
-			m_pParameters[j]->DataObjects_Set_Projection(Projection);
+			Get_Parameters(i)->DataObjects_Set_Projection(Projection);
 		}
 
 		return( true );
@@ -421,9 +433,9 @@ bool CSG_Tool::Get_Projection(CSG_Projection &Projection)	const
 
 	Parameters.DataObjects_Get_Projection(Projection);
 
-	for(int i=0; i<m_npParameters && !Projection.is_Okay(); i++)
+	for(int i=0; i<Get_Parameters_Count() && !Projection.is_Okay(); i++)
 	{
-		m_pParameters[i]->DataObjects_Get_Projection(Projection);
+		Get_Parameters(i)->DataObjects_Get_Projection(Projection);
 	}
 
 	return( Projection.is_Okay() );
@@ -482,26 +494,25 @@ int CSG_Tool::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *p
 //---------------------------------------------------------
 CSG_Parameters * CSG_Tool::Add_Parameters(const CSG_String &Identifier, const CSG_String &Name, const CSG_String &Description)
 {
-	CSG_Parameters	*pParameters;
-
-	m_pParameters	= (CSG_Parameters **)SG_Realloc(m_pParameters, (m_npParameters + 1) * sizeof(CSG_Parameters *));
-	pParameters		= m_pParameters[m_npParameters++]	= new CSG_Parameters();
+	CSG_Parameters *pParameters = new CSG_Parameters(); // (Name, Description, Identifier);
 
 	pParameters->Create(this, Name, Description, Identifier);
 	pParameters->Set_Callback_On_Parameter_Changed(&_On_Parameter_Changed);
-	pParameters->m_pTool	= this;
+	pParameters->m_pTool = this;
+
+	m_pParameters.Add(pParameters);
 
 	return( pParameters );
 }
 
 //---------------------------------------------------------
-CSG_Parameters * CSG_Tool::Get_Parameters(const CSG_String &Identifier)
+CSG_Parameters * CSG_Tool::Get_Parameters(const CSG_String &Identifier) const
 {
-	for(int i=0; i<m_npParameters; i++)
+	for(int i=0; i<Get_Parameters_Count(); i++)
 	{
-		if( m_pParameters[i]->Cmp_Identifier(Identifier) )
+		if( Get_Parameters(i)->Cmp_Identifier(Identifier) )
 		{
-			return( m_pParameters[i] );
+			return( Get_Parameters(i) );
 		}
 	}
 
@@ -511,7 +522,7 @@ CSG_Parameters * CSG_Tool::Get_Parameters(const CSG_String &Identifier)
 //---------------------------------------------------------
 bool CSG_Tool::Dlg_Parameters(const CSG_String &Identifier)
 {
-	CSG_Parameters	*pParameters	= Get_Parameters(Identifier);
+	CSG_Parameters *pParameters = Get_Parameters(Identifier);
 
 	if( pParameters && (!pParameters->is_Managed() || Dlg_Parameters(pParameters, Get_Name())) )
 	{
@@ -523,10 +534,20 @@ bool CSG_Tool::Dlg_Parameters(const CSG_String &Identifier)
 	return( false );
 }
 
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
 bool CSG_Tool::Dlg_Parameters(CSG_Parameters *pParameters, const CSG_String &Caption)
 {
-	return( pParameters ? SG_UI_Dlg_Parameters(pParameters, Caption) : false );
+	return( pParameters ? Dlg_Parameters(*pParameters, Caption) : false );
+}
+
+bool CSG_Tool::Dlg_Parameters(CSG_Parameters &Parameters, const CSG_String &Caption)
+{
+	return( SG_UI_Dlg_Parameters(&Parameters, Caption.is_Empty() ? Get_Name() : Caption) );
 }
 
 
@@ -539,9 +560,9 @@ void CSG_Tool::Set_Callback(bool bActive)
 {
 	Parameters.Set_Callback(bActive);
 
-	for(int i=0; i<m_npParameters; i++)
+	for(int i=0; i<Get_Parameters_Count(); i++)
 	{
-		m_pParameters[i]->Set_Callback(bActive);
+		Get_Parameters(i)->Set_Callback(bActive);
 	}
 }
 
@@ -550,9 +571,9 @@ bool CSG_Tool::Set_Manager(CSG_Data_Manager *pManager)
 {
 	Parameters.Set_Manager(pManager);
 
-	for(int i=0; i<m_npParameters; i++)
+	for(int i=0; i<Get_Parameters_Count(); i++)
 	{
-		m_pParameters[i]->Set_Manager(pManager);
+		Get_Parameters(i)->Set_Manager(pManager);
 	}
 
 	return( true );
@@ -579,7 +600,7 @@ bool CSG_Tool::Delete_Manager(bool bDetachData, bool bReset)
 
 	if( pManager && pManager != &SG_Get_Data_Manager() )
 	{
-		pManager->Delete_All(bDetachData);
+		pManager->Delete(bDetachData);
 
 		delete(pManager);
 
@@ -599,52 +620,27 @@ bool CSG_Tool::Delete_Manager(bool bDetachData, bool bReset)
 //---------------------------------------------------------
 bool CSG_Tool::Settings_Push(CSG_Data_Manager *pManager)
 {
-	if( m_Settings_Stack.Get_Value_Size() != sizeof(CSG_Parameters *) )
+	Parameters.Push(pManager);
+
+	for(int i=0; i<Get_Parameters_Count(); i++)
 	{
-		m_Settings_Stack.Create(sizeof(CSG_Parameters *));
+		Get_Parameters(i)->Push(pManager);
 	}
 
-	size_t	n	= m_Settings_Stack.Get_Size();
-
-	CSG_Parameters	**pP	= (CSG_Parameters **)m_Settings_Stack.Get_Array(n + 1 + m_npParameters);
-
-	if( pP )
-	{
-		pP[n++]	= new CSG_Parameters(Parameters); Parameters.Restore_Defaults(true); Parameters.Set_Manager(pManager);
-
-		for(int i=0; i<m_npParameters; i++)
-		{
-			pP[n++]	= new CSG_Parameters(*m_pParameters[i]); m_pParameters[i]->Restore_Defaults(true); m_pParameters[i]->Set_Manager(pManager);
-		}
-
-		return( true );
-	}
-
-	return( false );
+	return( true );
 }
 
 //---------------------------------------------------------
 bool CSG_Tool::Settings_Pop(void)
 {
-	CSG_Parameters	**pP	= (CSG_Parameters **)m_Settings_Stack.Get_Array();
+	Parameters.Pop();
 
-	if( pP && (int)m_Settings_Stack.Get_Size() >= 1 + m_npParameters )
+	for(int i=0; i<Get_Parameters_Count(); i++)
 	{
-		size_t	n	= m_Settings_Stack.Get_Size() - 1;
-
-		for(int i=m_npParameters-1; i>=0; i--, n--)
-		{
-			m_pParameters[i]->Assign_Values(pP[n]); m_pParameters[i]->Set_Manager(pP[n]->Get_Manager()); delete(pP[n]);
-		}
-
-		Parameters.Assign_Values(pP[n]); Parameters.Set_Manager(pP[n]->Get_Manager()); delete(pP[n]);
-
-		m_Settings_Stack.Set_Array(n);
-
-		return( true );
+		Get_Parameters(i)->Pop();
 	}
 
-	return( false );
+	return( true );
 }
 
 
@@ -800,16 +796,16 @@ bool CSG_Tool::Error_Fmt(const wchar_t *Format, ...)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Tool::DataObject_Add(CSG_Data_Object *pDataObject, bool bUpdate)
+bool CSG_Tool::DataObject_Add(CSG_Data_Object *pDataObject, bool bShow)
 {
 	if( Parameters.Get_Manager() )
 	{
 		Parameters.Get_Manager()->Add(pDataObject);
 	}
 
-	if( Parameters.Get_Manager() == &SG_Get_Data_Manager() )	// prevent that local data manager send their data objects to gui
+	if( Parameters.Get_Manager() == &SG_Get_Data_Manager() ) // prevent that local data manager send their data objects to gui
 	{
-		return( SG_UI_DataObject_Add(pDataObject, bUpdate) );
+		return( SG_UI_DataObject_Add(pDataObject, bShow ? SG_UI_DATAOBJECT_SHOW_MAP : SG_UI_DATAOBJECT_UPDATE) );
 	}
 
 	return( true );
@@ -1221,9 +1217,9 @@ bool CSG_Tool::Reset(bool bManager)
 		Reset_Manager();
 	}
 
-	for(int i=0; i<m_npParameters; i++)
+	for(int i=0; i<Get_Parameters_Count(); i++)
 	{
-		m_pParameters[i]->Restore_Defaults(true);
+		Get_Parameters(i)->Restore_Defaults(true);
 	}
 
 	return( Parameters.Restore_Defaults(true) );
@@ -1246,9 +1242,9 @@ bool CSG_Tool::Reset_Manager(void)
 */
 bool CSG_Tool::Reset_Grid_System(void)
 {
-	for(int i=0; i<m_npParameters; i++)
+	for(int i=0; i<Get_Parameters_Count(); i++)
 	{
-		m_pParameters[i]->Reset_Grid_System();
+		Get_Parameters(i)->Reset_Grid_System();
 	}
 
 	return( Parameters.Reset_Grid_System() );
@@ -1327,14 +1323,16 @@ CSG_String CSG_Tool::Get_Script(TSG_Tool_Script_Type Type, bool bHeader, bool bA
 {
 	switch( Type )
 	{
-	case TOOL_SCRIPT_CMD_SHELL            : return( _Get_Script_CMD           (      bHeader, bAllParameters, Type) );
-	case TOOL_SCRIPT_CMD_BATCH            : return( _Get_Script_CMD           (      bHeader, bAllParameters, Type) );
-	case TOOL_SCRIPT_CHAIN                : return( CSG_Tool_Chain::Get_Script(this, bHeader, bAllParameters      ) );
-	case TOOL_SCRIPT_PYTHON               : return( _Get_Script_Python        (      bHeader, bAllParameters      ) );
-	case TOOL_SCRIPT_PYTHON_WRAP_NAME     : return( _Get_Script_Python_Wrap   (      bHeader, true , false        ) );
-	case TOOL_SCRIPT_PYTHON_WRAP_NAME_CALL: return( _Get_Script_Python_Wrap   (      bHeader, true , true         ) );
-	case TOOL_SCRIPT_PYTHON_WRAP_ID       : return( _Get_Script_Python_Wrap   (      bHeader, false, false        ) );
-	case TOOL_SCRIPT_PYTHON_WRAP_ID_CALL  : return( _Get_Script_Python_Wrap   (      bHeader, false, true         ) );
+	case TOOL_SCRIPT_CMD_SHELL                 : return( _Get_Script_CMD           (      bHeader, bAllParameters, Type) );
+	case TOOL_SCRIPT_CMD_BATCH                 : return( _Get_Script_CMD           (      bHeader, bAllParameters, Type) );
+	case TOOL_SCRIPT_CMD_USAGE                 : return( _Get_Script_CMD_Usage     (                                   ) );
+	case TOOL_SCRIPT_CHAIN                     : return( CSG_Tool_Chain::Get_Script(this, bHeader, bAllParameters      ) );
+	case TOOL_SCRIPT_PYTHON                    : return( _Get_Script_Python        (      bHeader, bAllParameters      ) );
+	case TOOL_SCRIPT_PYTHON_WRAP_NAME          : return( _Get_Script_Python_Wrap   (      bHeader, true , false, false ) );
+	case TOOL_SCRIPT_PYTHON_WRAP_NAME_CALL     : return( _Get_Script_Python_Wrap   (      bHeader, true , true , true  ) );
+	case TOOL_SCRIPT_PYTHON_WRAP_NAME_CALL_FULL: return( _Get_Script_Python_Wrap   (      bHeader, true , true , false ) );
+	case TOOL_SCRIPT_PYTHON_WRAP_ID            : return( _Get_Script_Python_Wrap   (      bHeader, false, false, false ) );
+	case TOOL_SCRIPT_PYTHON_WRAP_ID_CALL       : return( _Get_Script_Python_Wrap   (      bHeader, false, true , false ) );
 	}
 
 	return( "" );
@@ -1559,6 +1557,144 @@ void CSG_Tool::_Get_Script_CMD(CSG_String &Script, CSG_Parameters *pParameters,
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_String CSG_Tool::_Get_Script_CMD_Usage(void)
+{
+	wxCmdLineParser Parser; Parser.SetSwitchChars("-");
+
+	_Get_Script_CMD_Usage(Get_Parameters(), Parser);
+
+	for(int i=0; i<Get_Parameters_Count(); i++)
+	{
+		_Get_Script_CMD_Usage(Get_Parameters(i), Parser);
+	}
+
+	wxString Usage = wxString::Format("\nUsage: saga_cmd %s %s %s", Get_Library().c_str(), Get_ID().c_str(),
+		Parser.GetUsageString().AfterFirst(' ').AfterFirst(' ')
+	);
+
+	CSG_String _Usage(&Usage);
+
+	return( _Usage );
+}
+
+//---------------------------------------------------------
+void CSG_Tool::_Get_Script_CMD_Usage(CSG_Parameters *pParameters, wxCmdLineParser &Parser)
+{
+	for(int i=0; i<pParameters->Get_Count(); i++)
+	{
+		CSG_Parameter *pParameter = pParameters->Get_Parameter(i);
+
+		//-------------------------------------------------
+		if( pParameter->is_DataObject() )	// reset data object parameters, avoids problems when tool is called more than once without un-/reloading
+		{
+			pParameter->Set_Value(DATAOBJECT_NOTSET);
+		}
+		else if( pParameter->is_DataObject_List() )
+		{
+			pParameter->asList()->Del_Items();
+		}
+
+		//-------------------------------------------------
+		if( pParameter->do_UseInCMD() == false )
+		{
+			continue;
+		}
+
+		wxString Description = pParameter->Get_Description(
+			PARAMETER_DESCRIPTION_NAME|PARAMETER_DESCRIPTION_TYPE|PARAMETER_DESCRIPTION_PROPERTIES, SG_T("\n\t")
+		).c_str();
+
+		Description.Replace("\xb", "");	// unicode problem: quick'n'dirty bug fix, to be replaced
+
+		wxString ID(pParameter->Get_CmdID().c_str());
+
+		if( pParameter->is_Input() || pParameter->is_Output() )
+		{
+			Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_NEEDS_SEPARATOR|wxCMD_LINE_PARAM_OPTIONAL);
+		}
+
+		//-------------------------------------------------
+		else if( pParameter->is_Option() && !pParameter->is_Information() )
+		{
+			switch( pParameter->Get_Type() )
+			{
+			case PARAMETER_TYPE_Parameters  :
+				_Get_Script_CMD_Usage(pParameter->asParameters(), Parser);
+				break;
+
+			case PARAMETER_TYPE_Bool        :
+				Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_Int         :
+				Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_NUMBER, wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_Data_Type   :
+			case PARAMETER_TYPE_Choice      :
+			case PARAMETER_TYPE_Choices     :
+			case PARAMETER_TYPE_Table_Field :
+			case PARAMETER_TYPE_Table_Fields:
+				Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_Double      :
+			case PARAMETER_TYPE_Degree      :
+				Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_DOUBLE, wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_Date        :
+				Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_DATE  , wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_Range       :
+				Parser.AddOption(ID + "_MIN", wxEmptyString, Description, wxCMD_LINE_VAL_DOUBLE, wxCMD_LINE_PARAM_OPTIONAL);
+				Parser.AddOption(ID + "_MAX", wxEmptyString, Description, wxCMD_LINE_VAL_DOUBLE, wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_Color       :
+				Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_Colors      :
+				Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_String      :
+			case PARAMETER_TYPE_Text        :
+			case PARAMETER_TYPE_FilePath    :
+				Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_FixedTable  :
+				Parser.AddOption(ID, wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);
+				break;
+
+			case PARAMETER_TYPE_Grid_System :
+				if( pParameter->Get_Children_Count() == 0 )
+				{
+					Parser.AddOption(ID + "_D"   , wxEmptyString, _TL("Cell Size"                          ), wxCMD_LINE_VAL_DOUBLE, wxCMD_LINE_PARAM_OPTIONAL);
+					Parser.AddOption(ID + "_X"   , wxEmptyString, _TL("Lower Left Center Cell X-Coordinate"), wxCMD_LINE_VAL_DOUBLE, wxCMD_LINE_PARAM_OPTIONAL);
+					Parser.AddOption(ID + "_Y"   , wxEmptyString, _TL("Lower Left Center Cell Y-Coordinate"), wxCMD_LINE_VAL_DOUBLE, wxCMD_LINE_PARAM_OPTIONAL);
+					Parser.AddOption(ID + "_NX"  , wxEmptyString, _TL("Number of Columns"                  ), wxCMD_LINE_VAL_NUMBER, wxCMD_LINE_PARAM_OPTIONAL);
+					Parser.AddOption(ID + "_NY"  , wxEmptyString, _TL("Number of Rows"                     ), wxCMD_LINE_VAL_NUMBER, wxCMD_LINE_PARAM_OPTIONAL);
+					Parser.AddOption(ID + "_FILE", wxEmptyString, _TL("Grid File"                          ), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);
+				}
+				break;
+
+			default:
+				break;
+			}
+		}
+	}
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
@@ -1841,28 +1977,28 @@ void CSG_Tool::_Get_Script_Python(CSG_String &Script, CSG_Parameters *pParameter
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_String CSG_Tool::_Get_Script_Python_Wrap(bool bHeader, bool bName, bool bCall)
+CSG_String CSG_Tool::_Get_Script_Python_Wrap(bool bHeader, bool bName, bool bCall, bool bOnlyNonDefaults)
 {
 	CSG_String Arguments, Description, Code;
 
 	for(int i=0; i<Parameters.Get_Count(); i++)	// add input that is not optional in 1st place
 	{
-		_Get_Script_Python_Wrap(Parameters[i], PARAMETER_INPUT         , Arguments, Description, Code, bCall && bHeader);
+		_Get_Script_Python_Wrap(Parameters[i], PARAMETER_INPUT         , Arguments, Description, Code, bCall && bHeader, bOnlyNonDefaults);
 	}
 
 	for(int i=0; i<Parameters.Get_Count(); i++) // add optional input in 2nd place
 	{
-		_Get_Script_Python_Wrap(Parameters[i], PARAMETER_INPUT_OPTIONAL, Arguments, Description, Code, bCall && bHeader);
+		_Get_Script_Python_Wrap(Parameters[i], PARAMETER_INPUT_OPTIONAL, Arguments, Description, Code, bCall && bHeader, bOnlyNonDefaults);
 	}
 
 	for(int i=0; i<Parameters.Get_Count(); i++) // add output
 	{
-		_Get_Script_Python_Wrap(Parameters[i], PARAMETER_OUTPUT        , Arguments, Description, Code, bCall && bHeader);
+		_Get_Script_Python_Wrap(Parameters[i], PARAMETER_OUTPUT        , Arguments, Description, Code, bCall && bHeader, bOnlyNonDefaults);
 	}
 
 	for(int i=0; i<Parameters.Get_Count(); i++) // add options
 	{
-		_Get_Script_Python_Wrap(Parameters[i], 0                       , Arguments, Description, Code, bCall && bHeader);
+		_Get_Script_Python_Wrap(Parameters[i], 0                       , Arguments, Description, Code, bCall && bHeader, bOnlyNonDefaults);
 	}
 
 	//---------------------------------------------------------
@@ -1891,26 +2027,21 @@ CSG_String CSG_Tool::_Get_Script_Python_Wrap(bool bHeader, bool bName, bool bCal
 		Name.Replace(" ", "_");
 	}
 
-	CSG_Strings _Description = SG_String_Tokenize(Get_Description(), "\n");
+	CSG_Strings _Description = SG_String_Tokenize(SG_HTML_Tag_Replacer(Get_Description()), "\n");
 
 	//---------------------------------------------------------
 	CSG_String Script;
 
 	if( bCall )
 	{
+		Script += "from PySAGA.tools import " + Get_Library() + "\n\n";
+
 		if( bHeader )
 		{
-			Script += "#! /usr/bin/env python\n";
-			Script += "from PySAGA import saga_api; from PySAGA.tools import " + Get_Library() + "\n\n";
 			Script += Code + "\n";
 		}
-		else
-		{
-			Script += "#! /usr/bin/env python\n";
-			Script += "from PySAGA.tools import " + Get_Library() + "\n\n";
-		}
 
-		Script += Get_Library() + '.' + Name + "(" + Arguments + ", Verbose=2)\n";
+		Script += Get_Library() + '.' + Name + "(" + Arguments + ")\n";
 	}
 	else
 	{
@@ -1948,7 +2079,7 @@ CSG_String CSG_Tool::_Get_Script_Python_Wrap(bool bHeader, bool bName, bool bCal
 }
 
 //---------------------------------------------------------
-bool CSG_Tool::_Get_Script_Python_Wrap(const CSG_Parameter &Parameter, int Constraint, CSG_String &Arguments, CSG_String &Description, CSG_String &Code, bool bCall, const CSG_String &Prefix)
+bool CSG_Tool::_Get_Script_Python_Wrap(const CSG_Parameter &Parameter, int Constraint, CSG_String &Arguments, CSG_String &Description, CSG_String &Code, bool bCall, bool bOnlyNonDefaults, const CSG_String &Prefix)
 {
 	if( Parameter.do_UseInCMD() == false
 	||  Parameter.is_Information()
@@ -1994,7 +2125,7 @@ bool CSG_Tool::_Get_Script_Python_Wrap(const CSG_Parameter &Parameter, int Const
 
 		for(int i=0; i<(*Parameter.asParameters()).Get_Count(); i++)
 		{
-			if( _Get_Script_Python_Wrap((*Parameter.asParameters())[i], Constraint, Arguments, Description, Code, bCall, ID) )
+			if( _Get_Script_Python_Wrap((*Parameter.asParameters())[i], Constraint, Arguments, Description, Code, bCall, bOnlyNonDefaults, ID) )
 			{
 				bResult = true;
 			}
@@ -2061,7 +2192,7 @@ bool CSG_Tool::_Get_Script_Python_Wrap(const CSG_Parameter &Parameter, int Const
 		}
 		else if( Parameter.is_Option() )
 		{
-			if( Parameter.is_Default() )
+			if( bOnlyNonDefaults && Parameter.is_Default() )
 			{
 				return( false );
 			}
